@@ -1,12 +1,12 @@
 /* Specific flags and argument handling of the Fortran front-end.
-   Copyright (C) 1997, 1999, 2000, 2001, 2002, 2003, 2004, 2005 Free
-   Software Foundation, Inc.
+   Copyright (C) 1997, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007,
+   2008  Free Software Foundation, Inc.
 
 This file is part of GCC.
 
 GNU CC is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
+the Free Software Foundation; either version 3, or (at your option)
 any later version.
 
 GNU CC is distributed in the hope that it will be useful,
@@ -15,9 +15,9 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with GNU CC; see the file COPYING.  If not, write to
-the Free Software Foundation, 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+along with GCC; see the file COPYING3.  If not see
+<http://www.gnu.org/licenses/>.  */
+
 /* This file is copied more or less verbatim from g77.  */
 /* This file contains a filter for the main `gcc' driver, which is
    replicated for the `gfortran' driver by adding this filter.  The purpose
@@ -51,6 +51,7 @@ Boston, MA 02111-1307, USA.  */
 
 #include "coretypes.h"
 #include "tm.h"
+#include "intl.h"
 
 #ifndef MATH_LIBRARY
 #define MATH_LIBRARY "-lm"
@@ -64,6 +65,20 @@ Boston, MA 02111-1307, USA.  */
 #define FORTRAN_LIBRARY "-lgfortran"
 #endif
 
+#ifdef HAVE_LD_STATIC_DYNAMIC
+#define ADD_ARG_LIBGFORTRAN(arg) \
+  { \
+    if (static_lib && !static_linking) \
+      append_arg ("-Wl,-Bstatic"); \
+    append_arg (arg); \
+    if (static_lib && !static_linking) \
+      append_arg ("-Wl,-Bdynamic"); \
+  }
+#else
+#define ADD_ARG_LIBGFORTRAN(arg) append_arg (arg);
+#endif
+
+
 /* Options this driver needs to recognize, not just know how to
    skip over.  */
 typedef enum
@@ -80,6 +95,8 @@ typedef enum
 				   -nodefaultlibs.  */
   OPTION_o,			/* Aka --output.  */
   OPTION_S,			/* Aka --assemble.  */
+  OPTION_static,		/* -static.  */
+  OPTION_static_libgfortran,	/* -static-libgfortran.  */
   OPTION_syntax_only,		/* -fsyntax-only.  */
   OPTION_v,			/* Aka --verbose.  */
   OPTION_version,		/* --version.  */
@@ -99,8 +116,6 @@ static void append_arg (const char *);
 static int g77_newargc;
 static const char **g77_newargv;
 
-const struct spec_function lang_specific_spec_functions[] = {{0,0}};
-
 /* --- This comes from gcc.c (2.8.1) verbatim: */
 
 /* This defines which switch letters take arguments.  */
@@ -161,7 +176,7 @@ lookup_option (Option *xopt, int *xskip, const char **xarg, const char *text)
     opt = OPTION_x, arg = text + 2;
   else
     {
-      if ((skip = WORD_SWITCH_TAKES_ARG (text + 1)) != 0)	/* See gcc.c.  */
+      if ((skip = WORD_SWITCH_TAKES_ARG (text + 1)) != 0)  /* See gcc.c.  */
 	;
       else if (!strcmp (text, "-fhelp"))	/* Really --help!! */
 	opt = OPTION_help;
@@ -170,6 +185,8 @@ lookup_option (Option *xopt, int *xskip, const char **xarg, const char *text)
 	opt = OPTION_nostdlib;
       else if (!strcmp (text, "-fsyntax-only"))
 	opt = OPTION_syntax_only;
+      else if (!strcmp (text, "-static-libgfortran"))
+	opt = OPTION_static_libgfortran;
       else if (!strcmp (text, "-dumpversion"))
 	opt = OPTION_version;
       else if (!strcmp (text, "-fversion"))	/* Really --version!! */
@@ -265,6 +282,12 @@ lang_specific_driver (int *in_argc, const char *const **in_argv,
   /* By default, we throw on the math library if we have one.  */
   int need_math = (MATH_LIBRARY[0] != '\0');
 
+  /* Whether we should link a static libgfortran.  */
+  int static_lib = 0;
+
+  /* Whether we need to link statically.  */
+  int static_linking = 0;
+
   /* The number of input and output files in the incoming arg list.  */
   int n_infiles = 0;
   int n_outfiles = 0;
@@ -279,7 +302,7 @@ lang_specific_driver (int *in_argc, const char *const **in_argv,
   g77_xargc = argc;
   g77_xargv = argv;
   g77_newargc = 0;
-  g77_newargv = (const char **) argv;
+  g77_newargv = CONST_CAST2 (const char **, const char *const *, argv);
 
   /* First pass through arglist.
 
@@ -323,6 +346,13 @@ lang_specific_driver (int *in_argc, const char *const **in_argv,
 	  library = 0;
 	  break;
 
+	case OPTION_static_libgfortran:
+	  static_lib = 1;
+	  break;
+
+	case OPTION_static:
+	  static_linking = 1;
+
 	case OPTION_l:
 	  ++n_infiles;
 	  break;
@@ -345,15 +375,13 @@ lang_specific_driver (int *in_argc, const char *const **in_argv,
 	  break;
 
 	case OPTION_version:
-	  printf ("\
-GNU Fortran 95 (GCC %s)\n\
-Copyright (C) 2006 Free Software Foundation, Inc.\n\
-\n\
-GNU Fortran comes with NO WARRANTY, to the extent permitted by law.\n\
+	  printf ("GNU Fortran %s%s\n", pkgversion_string, version_string);
+	  printf ("Copyright %s 2008 Free Software Foundation, Inc.\n\n",
+		  _("(C)"));
+	  printf (_("GNU Fortran comes with NO WARRANTY, to the extent permitted by law.\n\
 You may redistribute copies of GNU Fortran\n\
 under the terms of the GNU General Public License.\n\
-For more information about these matters, see the file named COPYING\n\
-", version_string);
+For more information about these matters, see the file named COPYING\n\n"));
 	  exit (0);
 	  break;
 
@@ -367,7 +395,7 @@ For more information about these matters, see the file named COPYING\n\
 	}
 
       /* This is the one place we check for missing arguments in the
-         program.  */
+	 program.  */
 
       if (i + skip < argc)
 	i += skip;
@@ -395,25 +423,25 @@ For more information about these matters, see the file named COPYING\n\
 	}
 
       if ((argv[i][0] == '-') && (argv[i][1] == 'M'))
-        {
-          char *p;
+	{
+	  char *p;
 
-          if (argv[i][2] == '\0')
-            {
-              p = xmalloc (strlen (argv[i + 1]) + 2);
-              p[0] = '-';
-              p[1] = 'J';
-              strcpy (&p[2], argv[i + 1]);
-              i++;
-            }
-          else
-            {
-              p = xmalloc (strlen (argv[i]) + 1);
-              strcpy (p, argv[i]);
-            }
-          append_arg (p);
-          continue;
-        }
+	  if (argv[i][2] == '\0')
+	    {
+	      p = XNEWVEC (char, strlen (argv[i + 1]) + 2);
+	      p[0] = '-';
+	      p[1] = 'J';
+	      strcpy (&p[2], argv[i + 1]);
+	      i++;
+	    }
+	  else
+	    {
+	      p = XNEWVEC (char, strlen (argv[i]) + 1);
+	      strcpy (p, argv[i]);
+	    }
+	  append_arg (p);
+	  continue;
+	}
 
       if ((argv[i][0] == '-') && (argv[i][1] != 'l'))
 	{
@@ -470,11 +498,16 @@ For more information about these matters, see the file named COPYING\n\
 		      append_arg (FORTRAN_INIT);
 		      use_init = 1;
 		    }
-		  append_arg (FORTRAN_LIBRARY);
+
+		  ADD_ARG_LIBGFORTRAN (FORTRAN_LIBRARY);
 		}
 	    }
 	  else if (strcmp (argv[i], FORTRAN_LIBRARY) == 0)
-	    saw_library = 1;	/* -l<library>.  */
+	    {
+	      saw_library = 1;	/* -l<library>.  */
+	      ADD_ARG_LIBGFORTRAN (argv[i]);
+	      continue;
+	    }
 	  else
 	    {			/* Other library, or filename.  */
 	      if (saw_library == 1 && need_math)
@@ -500,7 +533,9 @@ For more information about these matters, see the file named COPYING\n\
 	      append_arg (FORTRAN_INIT);
 	      use_init = 1;
 	    }
-	  append_arg (library);
+	  ADD_ARG_LIBGFORTRAN (library);
+	  /* Fall through.  */
+
 	case 1:
 	  if (need_math)
 	    append_arg (MATH_LIBRARY);
@@ -528,7 +563,7 @@ For more information about these matters, see the file named COPYING\n\
 
   if (verbose && g77_newargv != g77_xargv)
     {
-      fprintf (stderr, "Driving:");
+      fprintf (stderr, _("Driving:"));
       for (i = 0; i < g77_newargc; i++)
 	fprintf (stderr, " %s", g77_newargv[i]);
       fprintf (stderr, "\n");
@@ -538,6 +573,7 @@ For more information about these matters, see the file named COPYING\n\
   *in_argv = g77_newargv;
 }
 
+
 /* Called before linking.  Returns 0 on success and -1 on failure.  */
 int
 lang_specific_pre_link (void)	/* Not used for F77.  */
